/* 
 *  bitstream.c
 *
 *	Copyright (C) Aaron Holtzman - Dec 1999
 *
 *  This file is part of ac3dec, a free AC-3 audio decoder
 *	
 *  ac3dec is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  ac3dec is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>

#include "ac3.h"
#include "ac3_internal.h"
#include "bitstream.h"


uint8_t *buffer_start_ac3 = 0;
uint32_t ac3bits_left     = 0;
uint32_t ac3current_word;

static inline void
bitstream_fill_current()
{
	ac3current_word = *((uint32_t*)buffer_start_ac3)++;
	ac3current_word = swab32(ac3current_word);
}

//
// The fast paths for _get is in the
// bitstream.h header file so it can be inlined.
//
// The "bottom half" of this routine is suffixed _bh
//
// -ah
//

uint32_t
bitstream_get_bh_ac3(uint32_t num_bits)
{
	uint32_t result;

	num_bits -= ac3bits_left;
	result = (ac3current_word << (32 - ac3bits_left)) >> (32 - ac3bits_left);

	bitstream_fill_current();

	if(num_bits != 0)
		result = (result << num_bits) | (ac3current_word >> (32 - num_bits));
	
	ac3bits_left = 32 - num_bits;

	return result;
}

void
bitstream_init_ac3(uint8_t *start)
{
	//initialize the start of the buffer
	buffer_start_ac3 = start;
	ac3bits_left = 0;
}
